<?php

namespace Drupal\available_translation\Plugin\Block;

use Drupal\available_translation\Service\AvailableTranslationService;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an Available Translation block.
 *
 * @Block(
 *   id = "available_translation",
 *   admin_label = @Translation("Available Translation"),
 *   category = @Translation("Menus")
 * )
 */
class AvailableTranslationBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Service for building the available translation links.
   *
   * @var \Drupal\available_translation\Service\AvailableTranslationService
   */
  protected $service;

  /**
   * AvailableTranslationBlock constructor.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\available_translation\Service\AvailableTranslationService $service
   *   Service for building the available translation links.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, AvailableTranslationService $service) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->service = $service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('available_translation.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'template' => 'links',
      'delimiter' => ' | ',
      'position' => 'left',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $config = $this->getConfiguration();
    $default_config = $this->defaultConfiguration();

    $form['template'] = [
      '#type' => 'radios',
      '#title' => $this->t('Template'),
      '#options' => [
        'links' => $this->t("Links (theme_links)"),
        'available_translation_template' => $this->t("Available translation template"),
      ],
      '#default_value' => $config['template'] ?? $default_config['template'],
    ];

    $form['delimiter'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Delimiter'),
      '#description' => $this->t('Example: " | " for "Armenian | English | French"'),
      '#default_value' => $config['delimiter'] ?? $default_config['delimiter'],
      '#states' => [
        'visible' => [
          ':input[name="settings[template]"]' => ['value' => 'available_translation_template'],
        ],
      ],
    ];

    $form['position'] = [
      '#type' => 'radios',
      '#title' => $this->t('Position'),
      '#options' => [
        'left' => $this->t("Left"),
        'right' => $this->t("Right"),
      ],
      '#default_value' => $config['position'] ?? $default_config['position'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $values = $form_state->getValues();
    $this->setConfiguration([
      'template' => $values['template'],
      'delimiter' => $values['delimiter'],
      'position' => $values['position'],
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->getConfiguration();

    $build = [];
    if ($config["template"] === "links") {
      $build["links"] = [
        '#theme' => "links",
        '#links' => $this->service->getThemeLinks(),
        '#attributes' => [
          'class' => ['links', 'available-translation-' . $config['position']],
        ],
        "#attached" => [
          'library' => ['available_translation/available_translation'],
        ],
        '#cache' => [
          'max-age' => 0,
        ],
      ];
    }
    elseif ($config["template"] === "available_translation_template") {
      $build["available_translation_template"] = [
        '#theme' => "available_translation_template",
        '#text' => $this->service->getLinks($config['delimiter']),
        '#position' => $config['position'],
        "#attached" => [
          'library' => ['available_translation/available_translation'],
        ],
        '#cache' => [
          'max-age' => 0,
        ],
      ];
    }

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    if (!$account->hasPermission("access available_translation links")) {
      return AccessResult::forbidden();
    }
    return parent::blockAccess($account);
  }

}
