<?php

namespace Drupal\available_translation\Service;

use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Builds the available translation links.
 */
class AvailableTranslationService {
  use StringTranslationTrait;

  /**
   * Controls the lifecycle of requests.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  private $requestStack;

  /**
   * Language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  private $languageManager;

  /**
   * Available languages.
   *
   * @var array|\Drupal\Core\Language\LanguageInterface[]
   */
  private $availableLanguages = [];

  /**
   * Current request language.
   *
   * @var \Drupal\Core\Language\LanguageInterface
   */
  private $currentLanguage;

  /**
   * Node.
   *
   * @var \Drupal\node\Entity\Node
   */
  private $node;

  /**
   * Constructs an AvailableTranslationService object.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Controls the lifecycle of requests.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language manager.
   */
  public function __construct(RequestStack $request_stack, LanguageManagerInterface $language_manager) {
    $this->requestStack = $request_stack;
    $this->languageManager = $language_manager;
    $this->node = $this->requestStack->getCurrentRequest()->get('node');
    if ($this->node instanceof Node) {
      $this->availableLanguages = $this->node->getTranslationLanguages();
    }
    $this->currentLanguage = $this->languageManager->getCurrentLanguage();
  }

  /**
   * Builds the available translation links glued with given delimiter.
   *
   * @param string $delimiter
   *   Delimiter for the concatenation of the translation links.
   *
   * @return string
   *   Available translation links concatenated with delimiter
   */
  public function getLinks(string $delimiter) {
    $links = [];
    foreach ($this->availableLanguages as $language) {
      $options = ['language' => $language];
      if ($language->getId() === $this->currentLanguage->getId()) {
        $options['attributes']['class'] = "active";
      }
      $url = Url::fromRoute('entity.node.canonical', ['node' => $this->node->id()], $options);
      $link = Link::fromTextAndUrl($language->getName(), $url);
      $links[] = $link->toString();
    }
    $links = implode($delimiter, $links);
    $links = $links ? $this->t("Available in:") . " " . $links : $this->t("No translation available!");

    return $links;
  }

  /**
   * Builds the available translation links through theme_links template.
   *
   * @return array
   *   Array of links
   */
  public function getThemeLinks() {
    $links = [];
    foreach ($this->availableLanguages as $language) {
      $link_class = ['links__link'];
      if ($language->getId() === $this->currentLanguage->getId()) {
        $link_class[] = "active";
      }
      $links[] = [
        "title" => $language->getName(),
        "url" => Url::fromRoute(
          'entity.node.canonical', ['node' => $this->node->id()], ['language' => $language]
        ),
        'attributes' => ['class' => $link_class],
      ];
    }

    return $links;
  }

}
